ignoreModule = [
    "akamai_bot_lib",
    "user_identification_common",
    "blockcanary",
    "weaver",
    "testapp",
    "instrumentation_test",
    "graphql",
    "fingerprint",
    "dynamic_features",
    'kelontongapp',
    'dynamic-features',
    'projectFilesBackup3',
    'projectFilesBackup4',
    'dynamic-features-sellerapp',
    "buildconfig",
    "applink",
    "iris",
    "linker",
    "lint",
    "smart_recycler_diffutil_helper",
    "tools",
    "circular_view_pager",
    "common_network",
    "device_info",
    "inappupdate",
    "bottom_sheet_builder",
    "locationmanager",
    "logger",
    "permissionchecker",
    "remote_config",
    "remotesourcerequest",
    "track",
    "tracking_optimizer",
    "annotation_proc_central",
    "attachcommon",
    "common_payment",
    "encryption",
    "product_manage_common",
    "profilecommon",
    "screen_recorder",
    "seller_active_common",
    "tokopatch",
    "translator",
    "utils",
    "websocket",
    "youtube_common",
    "tkpdcore",
    "tkpddesign",
    "tkpdabstraction",
    ':features:content:play',
    ':features:content:play_widget',
    ':features:content:play_common',
    ':features:content:play_broadcaster',
    ':features:discovery:productcard',
    ':features:discovery:carousel_productcard'
]